@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -41,7 +41,7 @@ tp_f_update (crypt_hash_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -58,7 +58,7 @@ tp_f_update (crypt_hash_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -82,7 +82,7 @@ tp_f_reset (crypt_hash_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -105,7 +105,7 @@ tp_f_get_digest (crypt_hash_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -124,11 +124,11 @@ tp_f_get_hex_digest (crypt_hash_o *self, PyObject *)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_hex_digest ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_hex_digest ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -185,7 +185,7 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -202,7 +202,7 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
       catch (const std::exception& e)
         {
           Py_DECREF (ret);
-          PyErr_SetString (PyExc_Exception, e.what ());
+          mobius::py::set_runtime_error (e.what ());
           ret = nullptr;
         }
     }
@@ -218,7 +218,7 @@ static void
 tp_dealloc (crypt_hash_o *self)
 {
   delete self->obj;
-  self->ob_type->tp_free ((PyObject*) self);
+  Py_TYPE (self)->tp_free ((PyObject*) self);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -273,4 +273,5 @@ PyTypeObject crypt_hash_t =
   0,                                       		// tp_weaklist
   0,                                       		// tp_del
   0,                                       		// tp_version_tag
+  0,                                       		// tp_finalize
 };
