@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -20,50 +20,19 @@
 //! \brief  C++ API module wrapper
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+#include "module.h"
 #include <pymobius.h>
 #include "file.h"
 #include "folder.h"
 #include "path.h"
 #include "reader.h"
-#include "sector_reader.h"
+#include "sequential_reader_adaptor.h"
+#include "stream.h"
 #include "writer.h"
 #include "uri.h"
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief new_reader_from_sector_reader
-//! \param self function object
-//! \param args argument list
-//! \return Python object
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static PyObject *
-func_new_reader_from_sector_reader (PyObject *, PyObject *args)
-{
-  // parse input args
-  io_sector_reader_o * arg_sector_reader;
-
-  if (!PyArg_ParseTuple (args, "O!", &io_sector_reader_t, &arg_sector_reader))
-    return nullptr;
-
-  // execute C++ function
-  PyObject *ret = nullptr;
-
-  try
-    {
-      ret = PyMobius_IO_Reader_from_cpp (
-              mobius::io::new_reader_from_sector_reader (* (arg_sector_reader->obj))
-            );
-    }
-  catch (const std::exception& e)
-    {
-      PyErr_SetString (PyExc_Exception, e.what ());
-    }
-
-  // return value
-  return ret;
-}
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Module methods
+//! \brief Function prototypes
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *func_get_current_folder (PyObject *, PyObject *);
 PyObject *func_join_path (PyObject *, PyObject *);
@@ -77,6 +46,11 @@ PyObject *func_new_path_from_win (PyObject *, PyObject *);
 PyObject *func_new_uri_from_path (PyObject *, PyObject *);
 PyObject *func_to_win_path (PyObject *, PyObject *);
 
+namespace
+{
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Module methods
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyMethodDef module_methods[] =
 {
   {
@@ -139,12 +113,6 @@ static PyMethodDef module_methods[] =
     METH_VARARGS,
     "Create URI from path"
   },
-  {
-    "new_reader_from_sector_reader",
-    func_new_reader_from_sector_reader,
-    METH_VARARGS,
-    "Create reader from sector_reader object"
-  },
   {
     "to_win_path",
     func_to_win_path,
@@ -160,51 +128,42 @@ static PyMethodDef module_methods[] =
 };
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Module datatypes
+//! \brief Module definition structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static constexpr struct
+static PyModuleDef module_def =
 {
-  const char *name;
-  PyTypeObject *type;
-} types[] =
-
-{
-  {"file", &io_file_t},
-  {"folder", &io_folder_t},
-  {"path", &io_path_t},
-  {"reader", &io_reader_t},
-  {"sector_reader", &io_sector_reader_t},
-  {"writer", &io_writer_t},
-  {"uri", &io_uri_t},
+  PyModuleDef_HEAD_INIT,
+  "mobius.io",
+  "Mobius Forensic Toolkit mobius.io module",
+  -1,
+  module_methods,
+  nullptr,
+  nullptr,
+  nullptr,
+  nullptr
 };
 
+} // namespace
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Create module
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-PyObject *
+mobius::py::pymodule
 new_io_module ()
 {
-  // initialize module
-  PyObject* module = Py_InitModule3 (
-                       "mobius.io",
-                       module_methods,
-                       "Mobius Forensic Toolkit mobius.io module"
-                     );
+  // Initialize module
+  mobius::py::pymodule module (&module_def);
 
-  // build types
-  for (const auto& t : types)
-    {
-      if (PyType_Ready (t.type) < 0)
-        return nullptr;
-    }
+  // Add types
+  module.add_type ("file", &io_file_t);
+  module.add_type ("folder", &io_folder_t);
+  module.add_type ("path", &io_path_t);
+  module.add_type ("reader", &io_reader_t);
+  module.add_type ("sequential_reader_adaptor", &io_sequential_reader_adaptor_t);
+  module.add_type ("stream", &io_stream_t);
+  module.add_type ("writer", &io_writer_t);
+  module.add_type ("uri", &io_uri_t);
 
-  // add types
-  for (const auto& t : types)
-    {
-      Py_INCREF (t.type);
-      PyModule_AddObject (module, t.name, (PyObject *) t.type);
-    }
-
-  // return module
+  // Return module
   return module;
 }
