@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -32,7 +32,7 @@
 //! \return true/false
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 bool
-PyMobius_Io_Uri_Check (PyObject *pyobj)
+pymobius_io_uri_check (PyObject *pyobj)
 {
   return PyObject_IsInstance (pyobj, (PyObject *) &io_uri_t);
 }
@@ -43,7 +43,7 @@ PyMobius_Io_Uri_Check (PyObject *pyobj)
 //! \return new uri object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
-PyMobius_Io_Uri_from_cpp (mobius::io::uri obj)
+pymobius_io_uri_to_pyobject (mobius::io::uri obj)
 {
   PyObject *ret = _PyObject_New (&io_uri_t);
 
@@ -59,7 +59,7 @@ PyMobius_Io_Uri_from_cpp (mobius::io::uri obj)
 //! \return uri object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::io::uri
-PyMobius_Io_Uri_as_cpp (PyObject *pyobj)
+pymobius_io_uri_from_pyobject (PyObject *pyobj)
 {
   if (!PyObject_IsInstance (pyobj, (PyObject *) &io_uri_t))
     throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("object type must be mobius.io.uri"));
@@ -85,7 +85,7 @@ tp_f_get_value (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -110,7 +110,7 @@ tp_f_get_scheme (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -135,7 +135,7 @@ tp_f_get_authority (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -160,7 +160,7 @@ tp_f_get_path (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -185,7 +185,7 @@ tp_f_get_query (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -210,7 +210,7 @@ tp_f_get_fragment (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -235,7 +235,7 @@ tp_f_get_username (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -260,7 +260,7 @@ tp_f_get_password (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -285,7 +285,7 @@ tp_f_get_host (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -310,7 +310,7 @@ tp_f_get_port (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -331,11 +331,11 @@ tp_f_get_parent (io_uri_o *self, PyObject *)
 
   try
     {
-      ret = PyMobius_Io_Uri_from_cpp (self->obj->get_parent ());
+      ret = pymobius_io_uri_to_pyobject (self->obj->get_parent ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -360,7 +360,7 @@ tp_f_get_sibling_by_name (io_uri_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -369,11 +369,11 @@ tp_f_get_sibling_by_name (io_uri_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_Io_Uri_from_cpp (self->obj->get_sibling_by_name (arg_filename));
+      ret = pymobius_io_uri_to_pyobject (self->obj->get_sibling_by_name (arg_filename));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -398,7 +398,7 @@ tp_f_get_sibling_by_extension (io_uri_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -407,11 +407,11 @@ tp_f_get_sibling_by_extension (io_uri_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_Io_Uri_from_cpp (self->obj->get_sibling_by_extension (arg_extension));
+      ret = pymobius_io_uri_to_pyobject (self->obj->get_sibling_by_extension (arg_extension));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -436,7 +436,7 @@ tp_f_get_child_by_name (io_uri_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -445,11 +445,11 @@ tp_f_get_child_by_name (io_uri_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_Io_Uri_from_cpp (self->obj->get_child_by_name (arg_name));
+      ret = pymobius_io_uri_to_pyobject (self->obj->get_child_by_name (arg_name));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -474,7 +474,7 @@ tp_f_get_child_by_path (io_uri_o *self, PyObject *args)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_TypeError, e.what ());
+      mobius::py::set_invalid_type_error (e.what ());
       return nullptr;
     }
 
@@ -483,11 +483,11 @@ tp_f_get_child_by_path (io_uri_o *self, PyObject *args)
 
   try
     {
-      ret = PyMobius_Io_Uri_from_cpp (self->obj->get_child_by_path (arg_subpath));
+      ret = pymobius_io_uri_to_pyobject (self->obj->get_child_by_path (arg_subpath));
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -512,7 +512,7 @@ tp_f_get_filename (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -537,7 +537,7 @@ tp_f_get_extension (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -562,7 +562,7 @@ tp_f_is_empty (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -587,7 +587,7 @@ tp_f_is_relative (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -612,7 +612,7 @@ tp_f_is_absolute (io_uri_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -787,7 +787,7 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
       return nullptr;
     }
 
@@ -807,7 +807,7 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
       catch (const std::exception& e)
         {
           Py_DECREF (ret);
-          PyErr_SetString (PyExc_Exception, e.what ());
+          mobius::py::set_runtime_error (e.what ());
           ret = nullptr;
         }
     }
@@ -823,7 +823,7 @@ static void
 tp_dealloc (io_uri_o *self)
 {
   delete self->obj;
-  self->ob_type->tp_free ((PyObject*) self);
+  Py_TYPE (self)->tp_free ((PyObject*) self);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -878,4 +878,5 @@ PyTypeObject io_uri_t =
   0,                                       		// tp_weaklist
   0,                                       		// tp_del
   0,                                       		// tp_version_tag
+  0,                                       		// tp_finalize
 };
