@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -21,18 +21,19 @@
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include <pymobius.h>
+#include <pygil.h>
 #include "blob.h"
 #include "module.h"
 #include "io/reader.h"
 #include <mobius/io/bytearray_io.h>
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Check if object type is <i>blob</i>
+//! \brief Check if object is an instance of <i>mobius.os.win.dpapi.blob</i>
 //! \param pyobj Python object
 //! \return true/false
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 bool
-PyMobius_OS_Win_Dpapi_Blob_Check (PyObject *pyobj)
+pymobius_os_win_dpapi_blob_check (PyObject *pyobj)
 {
   return PyObject_IsInstance (pyobj, (PyObject *) &os_win_dpapi_blob_t);
 }
@@ -43,11 +44,9 @@ PyMobius_OS_Win_Dpapi_Blob_Check (PyObject *pyobj)
 //! \return new blob object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
-PyMobius_OS_Win_Dpapi_Blob_from_cpp (mobius::os::win::dpapi::blob obj)
+pymobius_os_win_dpapi_blob_to_pyobject (mobius::os::win::dpapi::blob obj)
 {
-  PyObject *ret = nullptr;
-
-  ret = _PyObject_New (&os_win_dpapi_blob_t);
+  PyObject *ret = _PyObject_New (&os_win_dpapi_blob_t);
 
   if (ret)
     ((os_win_dpapi_blob_o *) ret)->obj = new mobius::os::win::dpapi::blob (obj);
@@ -57,13 +56,16 @@ PyMobius_OS_Win_Dpapi_Blob_from_cpp (mobius::os::win::dpapi::blob obj)
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief Create <i>blob</i> C++ object from Python object
-//! \param pyobj Python object
+//! \param value Python object
 //! \return blob object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::os::win::dpapi::blob
-PyMobius_OS_Win_Dpapi_Blob_as_cpp (PyObject *pyobj)
+pymobius_os_win_dpapi_blob_from_pyobject (PyObject *value)
 {
-  return * (reinterpret_cast <os_win_dpapi_blob_o *>(pyobj)->obj);
+  if (!pymobius_os_win_dpapi_blob_check (value))
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("object must be an instance of mobius.os.win.dpapi.blob"));
+
+  return * (reinterpret_cast <os_win_dpapi_blob_o *>(value)->obj);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -78,11 +80,11 @@ tp_getter_revision (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_revision ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_revision ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -100,11 +102,11 @@ tp_getter_provider_guid (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_provider_guid ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_provider_guid ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -122,11 +124,11 @@ tp_getter_master_key_revision (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_master_key_revision ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_master_key_revision ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -144,11 +146,11 @@ tp_getter_master_key_guid (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_master_key_guid ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_master_key_guid ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -166,11 +168,11 @@ tp_getter_flags (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_flags ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_flags ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -188,11 +190,11 @@ tp_getter_description (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_description ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_description ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -210,11 +212,11 @@ tp_getter_cipher_id (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_cipher_id ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_cipher_id ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -232,11 +234,11 @@ tp_getter_key_length (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_key_length ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_key_length ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -258,7 +260,7 @@ tp_getter_salt (os_win_dpapi_blob_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -280,7 +282,7 @@ tp_getter_hmac_key (os_win_dpapi_blob_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -298,11 +300,11 @@ tp_getter_hash_id (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_hash_id ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_hash_id ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -320,11 +322,11 @@ tp_getter_hash_length (os_win_dpapi_blob_o *self)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_hash_length ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_hash_length ());
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -346,7 +348,7 @@ tp_getter_hmac_value (os_win_dpapi_blob_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -368,7 +370,7 @@ tp_getter_cipher_text (os_win_dpapi_blob_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -390,7 +392,7 @@ tp_getter_plain_text (os_win_dpapi_blob_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -412,7 +414,7 @@ tp_getter_signature (os_win_dpapi_blob_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -434,7 +436,7 @@ tp_getter_signature_data (os_win_dpapi_blob_o *self)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   return ret;
@@ -577,36 +579,33 @@ static PyObject *
 tp_f_test_key (os_win_dpapi_blob_o *self, PyObject *args)
 {
   // parse input args
-  const std::uint8_t * arg_key_buffer = nullptr;
-  Py_ssize_t arg_key_size;
-  const std::uint8_t * arg_entropy_buffer = nullptr;
-  Py_ssize_t arg_entropy_size;
-
-  if (!PyArg_ParseTuple (
-      args,
-      "s#|s#",
-      &arg_key_buffer,
-      &arg_key_size,
-      &arg_entropy_buffer,
-      &arg_entropy_size))
-    return nullptr;
-
-  mobius::bytearray arg_key (arg_key_buffer, arg_key_size);
+  mobius::bytearray arg_key;
   mobius::bytearray arg_entropy;
-  
-  if (arg_entropy_buffer)
-    arg_entropy = mobius::bytearray (arg_entropy_buffer, arg_entropy_size);
 
-  // execute C++ function
+  try
+    {
+      arg_key = mobius::py::get_arg_as_bytearray (args, 0);
+      arg_entropy = mobius::py::get_arg_as_bytearray (args, 1, mobius::bytearray ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
   PyObject *ret = nullptr;
 
   try
     {
-      ret = mobius::py::pybool_from_bool (self->obj->test_key (arg_key, arg_entropy));
+      auto rc = mobius::py::GIL () (
+                   self->obj->test_key (arg_key, arg_entropy)
+                );
+      ret = mobius::py::pybool_from_bool (rc);
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -623,39 +622,35 @@ static PyObject *
 tp_f_decrypt (os_win_dpapi_blob_o *self, PyObject *args)
 {
   // parse input args
-  const std::uint8_t * arg_key_buffer = nullptr;
-  Py_ssize_t arg_key_size;
-  const std::uint8_t * arg_entropy_buffer = nullptr;
-  Py_ssize_t arg_entropy_size;
-
-  if (!PyArg_ParseTuple (
-      args,
-      "s#|s#",
-      &arg_key_buffer,
-      &arg_key_size,
-      &arg_entropy_buffer,
-      &arg_entropy_size))
-    return nullptr;
-
-  mobius::bytearray arg_key (arg_key_buffer, arg_key_size);
+  mobius::bytearray arg_key;
   mobius::bytearray arg_entropy;
-  
-  if (arg_entropy_buffer)
-    arg_entropy = mobius::bytearray (arg_entropy_buffer, arg_entropy_size);
 
-  // execute C++ function
+  try
+    {
+      arg_key = mobius::py::get_arg_as_bytearray (args, 0);
+      arg_entropy = mobius::py::get_arg_as_bytearray (args, 1, mobius::bytearray ());
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
+
+  // execute C++ code
   PyObject *ret = nullptr;
 
   try
     {
-      ret = mobius::py::pybool_from_bool (self->obj->decrypt (arg_key, arg_entropy));
+      auto rc = mobius::py::GIL () (
+                   self->obj->decrypt (arg_key, arg_entropy)
+                );
+      ret = mobius::py::pybool_from_bool (rc);
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
-  // return value
   return ret;
 }
 
@@ -677,7 +672,7 @@ tp_f_is_decrypted (os_win_dpapi_blob_o *self, PyObject *)
     }
   catch (const std::exception& e)
     {
-      PyErr_SetString (PyExc_Exception, e.what ());
+      mobius::py::set_runtime_error (e.what ());
     }
 
   // return value
@@ -721,15 +716,17 @@ static PyObject *
 tp_new (PyTypeObject *type, PyObject *args, PyObject *)
 {
   // parse input args
-  PyObject *arg_reader = nullptr;
-  std::uint8_t *arg_data_buffer = nullptr;
-  Py_ssize_t arg_data_size = 0;
+  mobius::io::reader arg_reader;
 
-  if (!PyArg_ParseTuple (args, "s#", &arg_data_buffer, &arg_data_size) &&
-      !PyArg_ParseTuple (args, "O!", &io_reader_t, &arg_reader))
-    return nullptr;
-
-  PyErr_Clear ();
+  try
+    {
+      arg_reader = mobius::py::get_arg_as_cpp (args, 0, pymobius_io_reader_from_pyobject);
+    }
+  catch (const std::exception& e)
+    {
+      mobius::py::set_invalid_type_error (e.what ());
+      return nullptr;
+    }
 
   // create object
   os_win_dpapi_blob_o *ret = reinterpret_cast <os_win_dpapi_blob_o *> (type->tp_alloc (type, 0));
@@ -738,19 +735,11 @@ tp_new (PyTypeObject *type, PyObject *args, PyObject *)
     {
       try
         {
-          if (arg_reader)
-            ret->obj = new mobius::os::win::dpapi::blob (PyMobius_IO_Reader_as_cpp (arg_reader));
-
-          else
-            {
-              mobius::bytearray data (arg_data_buffer, arg_data_size);
-              auto reader = mobius::io::new_bytearray_reader (data);
-              ret->obj = new mobius::os::win::dpapi::blob (reader);
-            }
+          ret->obj = new mobius::os::win::dpapi::blob (arg_reader);
         }
       catch (const std::exception& e)
         {
-          PyErr_SetString (PyExc_Exception, e.what ());
+          mobius::py::set_runtime_error (e.what ());
           Py_DECREF (ret);
           ret = nullptr;
         }
@@ -767,7 +756,7 @@ static void
 tp_dealloc (os_win_dpapi_blob_o *self)
 {
   delete self->obj;
-  self->ob_type->tp_free ((PyObject*) self);
+  Py_TYPE (self)->tp_free ((PyObject*) self);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -822,4 +811,5 @@ PyTypeObject os_win_dpapi_blob_t =
   0,                                       		// tp_weaklist
   0,                                       		// tp_del
   0,                                       		// tp_version_tag
+  0,                                       		// tp_finalize
 };
