@@ -20,16 +20,19 @@
 //! \brief  C++ API mobius.system module wrapper
 //! \author Eduardo Aguiar
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+#include "module.h"
 #include <pymobius.h>
 #include "device.h"
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief functions prototypes
+//! \brief Function prototypes
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject* func_get_devices (PyObject *, PyObject *);
 
+namespace
+{
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief module methods
+//! \brief Module methods
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 static PyMethodDef module_methods[] =
 {
@@ -37,7 +40,7 @@ static PyMethodDef module_methods[] =
     "get_devices",
     func_get_devices,
     METH_VARARGS,
-    "get system's device list"
+    "Get system device list"
   },
   {
     NULL,
@@ -48,45 +51,35 @@ static PyMethodDef module_methods[] =
 };
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief module datatypes
+//! \brief Module definition structure
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static constexpr struct
+static PyModuleDef module_def =
 {
-  const char *name;
-  PyTypeObject *type;
-} types[] =
-
-{
-  {"device", &system_device_t},
+  PyModuleDef_HEAD_INIT,
+  "mobius.system",
+  "Mobius Forensic Toolkit mobius.system module",
+  -1,
+  module_methods,
+  nullptr,
+  nullptr,
+  nullptr,
+  nullptr
 };
 
+} // namespace
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create module
+//! \brief Create module
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-PyObject *
+mobius::py::pymodule
 new_system_module ()
 {
-  // initialize module mobius.system
-  PyObject* module = Py_InitModule3 (
-                       "mobius.system",
-                       module_methods,
-                       "Mobius Forensic Toolkit mobius.system module"
-                     );
+  // Initialize module
+  mobius::py::pymodule module (&module_def);
 
-  // build types
-  for (const auto& t : types)
-    {
-      if (PyType_Ready (t.type) < 0)
-        return nullptr;
-    }
+  // Add types
+  module.add_type ("device", &system_device_t);
 
-  // add types
-  for (const auto& t : types)
-    {
-      Py_INCREF (t.type);
-      PyModule_AddObject (module, t.name, (PyObject *) t.type);
-    }
-
-  // return module
+  // Return module
   return module;
 }
