@@ -1,6 +1,6 @@
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # Mobius Forensic Toolkit
-# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021 Eduardo Aguiar
+# Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -16,7 +16,6 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 import mobius
-import binascii
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Generic dataholder
@@ -42,15 +41,15 @@ class Entry (object):
     self.elapsed = None
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-# @brief get data as UTF-8 string (sanitize non UTF-8 strings)
+# @brief get data as string (sanitize non UTF-8 strings)
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 def get_utf8_string (data):
   try:
-    s = unicode (data, 'utf-8')
-  except UnicodeDecodeError, e:
-    s = unicode (data, 'iso-8859-1', errors='ignore')
+    s = data.decode ('utf-8')
+  except UnicodeDecodeError as e:
+    s = data.decode ('iso-8859-1', errors='ignore')
 
-  return s.encode ('utf-8')
+  return s
 
 # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 # @brief Decode PBTHash_XXX.dat file
@@ -64,14 +63,14 @@ def decode (f):
   # check version
   decoder = mobius.decoder.data_decoder (reader)
   version = decoder.get_uint8 ()
-  
+
   if version != 1:
     return
 
   # create entry object
   entry = Entry ()
   entry.db_version = version
-  entry.hash_sha1 = binascii.hexlify (decoder.get_bytearray_by_size (20))
+  entry.hash_sha1 = mobius.encoder.hexstring (decoder.get_bytearray_by_size (20))
   entry.pbthash_path = f.path.replace ('/', '\\')
   entry.pbthash_creation_time = f.creation_time
   entry.pbthash_last_modification_time = f.modification_time
@@ -82,22 +81,22 @@ def decode (f):
   state = decoder.get_uint8 ()
   entry.flag_paused = (state == 1)
   entry.flag_seeding = (state == 2)
-  
+
   # get info
   entry.size = decoder.get_uint64_le ()
   entry.piece_length = decoder.get_uint32_le ()
   entry.bytes_downloaded = decoder.get_uint64_le ()
   entry.bytes_uploaded = decoder.get_uint64_le ()
   entry.num_pieces = decoder.get_uint32_le ()
-  
+
   # get pieces
   # @see BitTorrentDb_load - BitTorrent/BitTorrentDlDb.pas (line 163)
   entry.pieces = []
 
-  for i in xrange (entry.num_pieces):
+  for i in range (entry.num_pieces):
     piece = dataholder ()
     piece.idx = i
-    piece.hash_sha1 = binascii.hexlify (decoder.get_bytearray_by_size (20))
+    piece.hash_sha1 = mobius.encoder.hexstring (decoder.get_bytearray_by_size (20))
     piece.flag_checked = decoder.get_uint8 () == 1
     entry.pieces.append (piece)
 
@@ -118,7 +117,7 @@ def decode (f):
   # @see BitTorrentDb_load - BitTorrent/BitTorrentDlDb.pas (line 211)
   entry.files = []
 
-  for i in xrange (entry.num_files):
+  for i in range (entry.num_files):
     fl = dataholder ()
     fl.idx = i
     fl.size = decoder.get_uint64_le ()
@@ -131,7 +130,7 @@ def decode (f):
       siz = decoder.get_uint16_le ()
 
     fl.name = decoder.get_string_by_size (siz)
-    
+
     # @see BitTorrentDb_load - BitTorrent/BitTorrentDlDb.pas (line 241)
     if entry.num_files == 1:
       fl.path = entry.path
@@ -140,7 +139,7 @@ def decode (f):
 
     fl.name = fl.name.rsplit ('\\')[-1]
     entry.files.append (fl)
-  
+
   # decode tags
   # @see BitTorrentDb_load - BitTorrent/BitTorrentDlDb.pas (line 296)
   pos = decoder.tell ()
@@ -151,7 +150,7 @@ def decode (f):
     tag_id = decoder.get_uint8 ()
     tag_size = decoder.get_uint16_le ()
     pos = pos + 3 + tag_size
-    
+
     if tag_id == 1:
       entry.name = get_utf8_string (decoder.get_string_by_size (tag_size))
 
@@ -161,13 +160,13 @@ def decode (f):
 
     elif tag_id == 3:
       entry.comment = get_utf8_string (decoder.get_string_by_size (tag_size))
-      
+
     elif tag_id == 4:
       entry.db_time = decoder.get_unix_datetime ()
 
     elif tag_id == 5:
       entry.start_time = decoder.get_unix_datetime ()
-      
+
     elif tag_id == 7:
       entry.elapsed = decoder.get_uint32_le ()
 
